#!/usr/bin/env php
<?php
/**
 * Copyright (C) 2005-2015 it-novum GmbH
 *
 * This program is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License version 2 as published 
 * by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
 * See the GNU General Public License for more details.
 **/

require_once "core.php";
bootup();

$Client = new phpNSTAClient();
$Client->work();

class phpNSTAClient{

	private $worker;
	private $bulkTransmit = true;
	private $compression = false;
	
	private $Bulk = null;

	public function __construct(){
		$this->bindSignalHandler();
		$this->worker = new GearmanWorker();

		$this->worker->addServer(Config::val('GEARMAN', 'host'), Config::val('GEARMAN', 'port'));
		$this->worker->addOptions(GEARMAN_WORKER_NON_BLOCKING);
		$this->worker->setTimeout(Config::val('GEARMAN', 'timeout'));

		$this->worker->addFunction("NAGIOS_COMMAND", array($this, "ParseNagiosCommands"));
		$this->worker->addFunction("SET_SYSTEMTIME", array($this, "SetSystemTime"));

		$this->bulkTransmit = Config::val('GEARMAN', 'bulk_transmit');
		if($this->bulkTransmit === true){
			Logger::log('Bulk transmission enabled', 2);
			$this->worker->addFunction("statusngin_ochp", array($this, "ParseStatusengineOchp"));
			$this->worker->addFunction("statusngin_ocsp", array($this, "ParseStatusengineOcsp"));
			$this->worker->addFunction("phpnsta_custom_input", array($this, "ProcessCustomInput"));
			
			Logger::log('Checking available compression algorithms', 2);
			$compression = Config::val('GEARMAN', 'compression');
			Logger::log('Compression is set to "'.$compression.'"', 2);
			
			$_compression = false;
			if($compression === 'auto'){
				if(function_exists('gzcompress')){
					$_compression = 'gz';
				}else{
					Logger::log('gzcompress is not available!', 2);
				}
				if(function_exists('bzcompress')){
					$_compression = 'bz2';
				}else{
					Logger::log('bzcompress is not available!', 2);
				}
				Logger::log('Used compression is "'.$_compression.'"', 2);
			}else{
				if($compression === 'bz2'){
					if(!function_exists('bzcompress')){
						Logger::log('bzcompress is not available! No fallback due to config!', 2);
					}else{
						$_compression = $compression;
					}
				}
				if($compression === 'gz'){
					if(!function_exists('gzcompress')){
						Logger::log('gzcompress is not available! No fallback due to config!', 2);
					}else{
						$_compression = $compression;
					}
				}
			}
			$this->compression = $_compression;
			Logger::log('Compression algorithm for bulk transmission is "'.$this->compression.'"', 2);
			$this->Bulk = new Bulk($_compression);
		}

		if(!$this->worker->echo('Ping')){
			Logger::log('Gearman Job Server "'.Config::val('GEARMAN', 'host').':'.Config::val('GEARMAN', 'port').'" not reachable', 1);
		}
	}
	
	public function work(){
		while(true){
			pcntl_signal_dispatch();
			$this->worker->work();
			@$this->worker->wait();

			if($this->bulkTransmit === true){
				if($this->Bulk->flushRequiredForHosts()){
					$this->Bulk->flushHosts();
				}
				if($this->Bulk->flushRequiredForServices()){
					$this->Bulk->flushServices();
				}
				if($this->Bulk->flushRequiredForCustomData()){
					$this->Bulk->flushCustom();
				}
			}

			$rc = $this->worker->returnCode();
			if($rc != GEARMAN_SUCCESS){
				if($rc == GEARMAN_NO_JOBS || $rc == GEARMAN_IO_WAIT || $rc == GEARMAN_NO_ACTIVE_FDS || $rc == GEARMAN_TIMEOUT){
					continue;
				}
				Logger::log('Failed to process gearman Job!', 1);
				Logger::log($this->worker->error(), 1);
			}
		}
	}



	/**
	 * Function: ParseNagiosCommands
	 *
	 * Übergibt die Daten, welche aus dem Gearman Server kommen, an Nagios
	 *
	 * @param  object $job  Gearman $job Object
	 * @return void
	 * @author Daniel Ziegler <daniel.ziegler@it-novum.com>
	 * @since 1.0
	 *
	 */
	public function ParseNagiosCommands($job){
		$param = $job->workload();
		//Check if Nagios is running
		Logger::log('Receiving external Nagios command', 1);
		if(file_exists(Config::val('NAGIOS', 'nagios_cmd'))){
			$file = fopen(Config::val('NAGIOS', 'nagios_cmd'), 'a+');
			fwrite($file, $param);
			fclose($file);
		}
		unset($param);
	}

	/**
	 * Function: SetSystemTime
	 *
	 * Setzt die Neue Uhrzeit, welche durch den Masterserver übermittelt wird
	 *
	 * @param  object $job  Gearman $job Object
	 * @return void
	 * @author Daniel Ziegler <daniel.ziegler@it-novum.com>
	 * @since 1.1
	 *
	 */
	public function SetSystemTime($job){
		Logger::log('Receiving new Timestamp from master server ('.$job->workload().')', 1);
		if(is_numeric($job->workload())){
			exec("date -s @".$job->workload(), $output, $returncode);
		}
	
		if($returncode != 0){
			Logger::log("Can't set new system time!", 1);
			Logger::dump($output);
		}
	}

	/**
	 * Function: ParseStatusengineOchp
	 *
	 * Die Daten werden nicht über das Klassische OCHP geliefert, sondern
	 * von Statusengine enable_ochp=1 bereitgestellt
	 *
	 * @param object $job Gearman $job Object
	 * @return void
	 * @author Daniel Ziegler <daniel.ziegler@it-novum.com>
	 * @since 1.6
	 *
	 */
	public function ParseStatusengineOchp($job){
		$workload = json_decode($job->workload());
		//Logger::dump($workload);
	
		$data = array(
			'TYPE' => 'HOST',
			'TIMET' => $workload->hostcheck->start_time,
			'HOSTNAME' => $workload->hostcheck->host_name,
			'HOSTSTATEID' => $workload->hostcheck->state,
			'HOSTOUTPUT' => $workload->hostcheck->output,
		
			//Mod_Gearman additional fields
			'start_time' => $workload->hostcheck->start_time,
			'end_time' => $workload->hostcheck->end_time,
			'latency' => $workload->hostcheck->latency,
		);
	
		unset($workload);
		$this->Bulk->addHostRecord($data);
	}

	/**
	 * Function: ParseStatusengineOcsp
	 *
	 * Die Daten werden nicht über das Klassische OCSP geliefert, sondern
	 * von Statusengine enable_ocsp=1 bereitgestellt
	 *
	 * @param object $job Gearman $job Object
	 * @return void
	 * @author Daniel Ziegler <daniel.ziegler@it-novum.com>
	 * @since 1.6
	 *
	 */
	public function ParseStatusengineOcsp($job){
		$workload = json_decode($job->workload());
	
		$data = array(
			'TYPE' => 'SERVICE',
			'TIMET' => $workload->servicecheck->start_time,
			'HOSTNAME' => $workload->servicecheck->host_name,
			'SERVICEDESC' => $workload->servicecheck->service_description,
			'SERVICESTATEID' => $workload->servicecheck->state,
			'SERVICEOUTPUT' => $workload->servicecheck->output,
			'SERVICEPERFDATA' => $workload->servicecheck->perf_data,
			'LONGSERVICEOUTPUT' => $workload->servicecheck->long_output,
	
			//Mod_Gearman additional fields
			'start_time' => $workload->servicecheck->start_time,
			'end_time' => $workload->servicecheck->end_time,
			'latency' => $workload->servicecheck->latency,
		);
	
		$this->Bulk->addServiceRecord($data);
	}
	
	/**
	 * Function: ProcessCustomInput
	 *
	 * Verarbeitet vom Kunden eigene Daten, welche auf das Master
	 * System übertragen werden sollen
	 *
	 * @param object $job Gearman $job Object
	 * @return void
	 * @author Daniel Ziegler <daniel.ziegler@it-novum.com>
	 * @since 1.10
	 *
	 */
	public function ProcessCustomInput($job){
		$input = base64_encode($job->workload());
		$this->Bulk->addCustomRecord($input);
	}
	
	public function bindSignalHandler(){
		pcntl_signal(SIGTERM, array($this, "sig_handler"));
		pcntl_signal(SIGINT,  array($this, "sig_handler"));
	}
	
	public function sig_handler($signo){
		switch($signo){
			case SIGTERM:
			case SIGINT:
				if($this->bulkTransmit === true){
					Logger::log("Force bulk flush because of exit", 2);
					$this->Bulk->flushHosts();
					$this->Bulk->flushServices();
					$this->Bulk->flushCustom();
				}
				Logger::log("bye...", 1);
				exit(0);
			break;
			
			default:
				Logger::log("Caught unknown signal...", 1);
				break;
		}
		$this->bindSignalHandler();
	}
}
